from datetime import datetime
from db_base import Base
from sqlalchemy import Column, Integer, String, DateTime, Numeric, ForeignKey

class ErrataType:
    BugFixAdvisory,SecurityAdvisory,ProductEnhancementAdvisory = range(3)
    
    @staticmethod
    def get_label(value):
        labels = {
        ErrataType.BugFixAdvisory: 'Bug Fix Advisory',
        ErrataType.SecurityAdvisory: 'Security Advisory',
        ErrataType.ProductEnhancementAdvisory: 'Product Enhancement Advisory'
        }
        return labels[value]
    
class ErrataSeverity:
    Critical, Important, Moderate, Low = range(4)
    
    @staticmethod
    def get_label(value):
        labels = {
        ErrataSeverity.Important: 'Important',
        ErrataSeverity.Moderate: 'Moderate',
        ErrataSeverity.Low: 'Low',
        ErrataSeverity.Critical: 'Critical',
        None: '(No severity supplied)'
        }
        return labels[value]

class ErrataItem(Base):
    __tablename__ = 'notified_advisories'
    id = Column(Integer, primary_key=True)
    advisory_id = Column(String)
    timestamp = Column(DateTime)
    
    def __init__(self,advisory_id,type,severity,architectures,releases,packages,references):
        self.advisory_id = advisory_id
        self.type = type
        self.severity = severity
        self.architectures = architectures
        self.releases = releases
        self.packages = packages
        self.references = references
        self.timestamp = datetime.today()



import operator
import re
from rpmUtils.miscutils import compareEVR

class PackageChecker:
    def __init__(self, errata_fetcher, package_fetcher, os_fetcher):
        self.errata_fetcher = errata_fetcher
        self.package_fetcher = package_fetcher
        self.os_fetcher = os_fetcher

    def _advisoryPackageMeantForCurrentOs(self, advisory_package):
        os_version = self.os_fetcher.get_top_level_version()
        regex = r'.*el'+os_version+'.*'
        return re.match(regex, advisory_package['release']) != None

    def _compareAdvisoryAgainstInst(self,advisory_package,installed_package):
        return compareEVR( ('', advisory_package['version'], advisory_package['release']), ('', installed_package.version, installed_package.release))

    def match_advisory_against_installed(self,advisory_package,current_installed):
        installed_versions = filter(lambda inst: advisory_package['name'] == inst.name, current_installed)
        if not self._advisoryPackageMeantForCurrentOs(advisory_package):
            return []
        # Deal with cases where both old and new kernel packages are installed
        still_vulnerable = all(self._compareAdvisoryAgainstInst(advisory_package, inst) > 0 for inst in installed_versions)
        return installed_versions if still_vulnerable else []

    def findAdvisoriesOnInstalledPackages(self):
        os_version = self.os_fetcher.get_top_level_version()
        advisories = self.errata_fetcher.get_errata()
        current_installed = self.package_fetcher.fetch_installed_packages()
        results = []
        for advisory in advisories:
            installed_package_match = map(lambda advisory_package: self.match_advisory_against_installed(advisory_package,current_installed),advisory.packages)
            # Need to flatten
            installed_package_match = reduce(operator.add, installed_package_match)
            if len(installed_package_match) > 0 and (os_version in advisory.releases):
                results.append({'advisory': advisory, 'installed_packages':installed_package_match})
        return results




from datetime import datetime
from db_base import Base
from sqlalchemy import Column, Integer, String, DateTime, Numeric, ForeignKey
from rpmUtils.miscutils import compareEVR

class Package(Base):
    __tablename__ = 'notified_packages'
    id = Column(Integer, primary_key=True)
    name = Column(String)
    version = Column(String)
    release = Column(String)
    arch = Column(String)
    repository = Column(String)
    timestamp = Column(DateTime)
    
    def __init__(self,name,version,release,arch,repository):
        self.name = name
        self.version = version
        self.release = release
        self.arch = arch
        self.repository = repository
        self.timestamp = datetime.today()
        
    def compare_evr(self, other_package):            
        return compareEVR( ('', self.version, self.release), ('', other_package.version, other_package.release))
        
    def __str__(self):
        return "Package %s-%s-%s" % (self.name, self.version, self.release)
        
    def __repr__(self):
        return self.__str__()


# coding: utf8

from errata_item import *
import urllib2
import contextlib
from os.path import exists
from ConfigParser import RawConfigParser

from xml.etree import ElementTree as et
from package_parser import PackageParser

YUM_CONF = "/etc/yum.conf"
ERRATA_URL = "https://cefs.steve-meier.de/errata.latest.xml"


def get_opener():
    default_opener = urllib2.build_opener()
    if not exists(YUM_CONF):
        return default_opener
    config = RawConfigParser()
    config.read(YUM_CONF)
    if not config.has_section('main'):
        return default_opener
    if not config.has_option('main', 'proxy'):
        return default_opener
    proxy = {}
    url = config.get('main', 'proxy').strip()
    if not url:
        return default_opener
    http_proxy_handler = urllib2.ProxyHandler({'http': url, 'https': url})
    # urllib2 can open HTTPS ressources through a proxy since python 2.6.3
    # should be OK on Centos OS (Python 2.6.6)
    if config.has_option('main', 'proxy_username') and config.has_option(
            'main', 'proxy_password'):
        username = config.get('main', 'proxy_username').strip()
        password = config.get('main', 'proxy_password').strip()
        password_manager = urllib2.HTTPPasswordMgrWithDefaultRealm()
        password_manager.add_password(None, url, username, password)
        proxy_auth_handler = urllib2.ProxyBasicAuthHandler(password_manager)
        return urllib2.build_opener(http_proxy_handler, proxy_auth_handler)
    return urllib2.build_opener(http_proxy_handler)


class ErrataParser:

    def getType(self, theType):
        mapping = {
            'Bug Fix Advisory': ErrataType.BugFixAdvisory,
            'Security Advisory': ErrataType.SecurityAdvisory,
            'Product Enhancement Advisory':
            ErrataType.ProductEnhancementAdvisory
        }
        try:
            return mapping[theType]
        except KeyError:
            print "Do not understand mapping for type %s" % (theType)
            raise

    def getSeverity(self, theSeverity):
        if theSeverity == None:
            return None
        mapping = {
            'Important': ErrataSeverity.Important,
            'Moderate': ErrataSeverity.Moderate,
            'Low': ErrataSeverity.Low,
            'Critical': ErrataSeverity.Critical
        }
        try:
            return mapping[theSeverity]
        except KeyError:
            print "Do not understand mapping for severity %s" % (theSeverity)
            raise

    def parseSingleItem(self, node):
        try:
            if node.tag == 'meta':
                return None
            # Sometimes empty elements are in there
            if 'type' not in node.attrib:
                return None
            the_type = self.getType(node.attrib['type'])
            severity = self.getSeverity(node.attrib.get('severity'))
            architectures = map(lambda x: x.text, node.findall('os_arch'))
            releases = map(lambda x: x.text, node.findall('os_release'))
            packages = map(lambda x: PackageParser.parsePackage(x.text),
                           node.findall('packages'))
            references = node.attrib.get('references').split(' ')
            return ErrataItem(node.tag, the_type, severity, architectures,
                              releases, packages, references)
        except:
            print "Problem while parsing node %s" % (node)
            raise

    def parse(self, xml_str):
        dom = et.fromstring(xml_str)
        assert dom.tag == 'opt', "Expecting doc root to be opt but was %s" % (
            doc.localName)
        result = map(lambda x: self.parseSingleItem(x), dom)
        result = list(filter(lambda x: x != None, result))
        return result


class ErrataFetcher:

    def get_errata(self):
        with contextlib.closing(get_opener().open(ERRATA_URL)) as response:
            xml = response.read()
        parser = ErrataParser()
        return parser.parse(xml)




from annoyance_fetcher import AnnoyanceFetcher
from db_session_fetcher import db_session_fetcher
from package_fetcher import *
from package_checker import *
from errata_fetcher import *
from os_version_fetcher import *
from mockable_execute import *

import json

class ReportProducer:
    def __init__(self,
                 repos_to_exclude_list,
                 repos_to_include_list,
                 skip_old_notices,
                 db_file_path,
                 pkg_fetcher=None,
                 checker=None,
                 annoyance_fetcher=None,
                 db_session_fetch=None,
                 include_depends_on=None):
        self.executor = MockableExecute()
        self.pkg_fetcher = pkg_fetcher or PackageFetcher(ChangeLogParser(), self.executor, repos_to_exclude_list, repos_to_include_list)
        self.checker = checker or PackageChecker(ErrataFetcher(), self.pkg_fetcher, OsVersionFetcher())
        self.annoyance_fetcher = annoyance_fetcher or AnnoyanceFetcher()
        self.db_session_fetch = db_session_fetch or db_session_fetcher(db_path=db_file_path)
        self.annoyance_check = None
        self.skip_old_notices = skip_old_notices
        self.include_depends_on = include_depends_on
        
    def _get_sorted_relevant_advisories(self):
        security_advisories = filter(lambda adv:adv['advisory'].type == ErrataType.SecurityAdvisory,self.checker.findAdvisoriesOnInstalledPackages())
        if self.skip_old_notices:
            only_advisories = map(lambda advpkg: advpkg['advisory'], security_advisories)
            self.annoyance_check.remove_old_advisories(only_advisories)
            security_advisories = filter(lambda advpkg: self.annoyance_check.is_advisory_alert_necessary(advpkg['advisory']), security_advisories)

        security_advisories = sorted(security_advisories, key=lambda adv: adv['advisory'].severity)

        return security_advisories

    def _add_advisories_to_email(self, security_advisories, email_body):
        if len(security_advisories) > 0:
            email_body += u'The following security advisories exist for installed packages:\n\n'
        for advisory_and_package in security_advisories:
            advisory = advisory_and_package['advisory']
            email_body += u"Advisory ID: %s\n" % (advisory.advisory_id)
            severity_label = ErrataSeverity.get_label(advisory.severity)
            email_body += u"Severity: %s\n" % (severity_label)
            associated_package_labels = map(lambda pkg: "* %s-%s-%s" % (pkg.name, pkg.version, pkg.release),advisory_and_package['installed_packages'])
            # Remove dupes
            associated_package_labels = list(set(associated_package_labels))
            packages_flat = u"\n".join(associated_package_labels)
            email_body += u"Packages:\n%s\n" % (packages_flat)
            references = map(lambda ref: u"* %s" % (ref), advisory.references)
            references_flat = u"\n".join(references)
            email_body += u"References:\n%s\n\n" % (references_flat)

        return email_body

    def _get_advisories_as_json(self, security_advisories):
        records = []
        for advisory_and_package in security_advisories:

            record = {}

            advisory = advisory_and_package['advisory']
            record["advisory_id"] = advisory.advisory_id
            record["severity"] = ErrataSeverity.get_label(advisory.severity)
            associated_package_labels = map(lambda pkg: "%s-%s-%s" % (pkg.name, pkg.version, pkg.release), advisory_and_package['installed_packages'])
            record["packages"] = list(set(associated_package_labels))

            records.append(record)

        return records

    def _get_general_updates(self):
        general_updates = self.pkg_fetcher.get_package_updates()
        if self.skip_old_notices:
            general_updates = filter(lambda pkg: self.annoyance_check.is_package_alert_necessary(pkg), general_updates)
            for update in general_updates:
                self.annoyance_check.remove_old_alerts_for_package(update)

        return sorted(general_updates, key=lambda pkg: pkg.name)

    def _add_general_updates_to_email(self, general_updates, email_body):
        if len(general_updates) > 0:
            email_body += u"The following packages are available for updating:\n\n"

        for update in general_updates:
            email_body += u"%s-%s-%s from %s\n" % (update.name, update.version, update.release, update.repository)

        if self.include_depends_on:
            if len(general_updates) > 0:
                email_body += u"\n"
            for update in general_updates:
                depends_on = self.pkg_fetcher.get_what_depends_on(update.name)
                if len(depends_on) > 0:
                    email_body += u"These packages depend on %s:\n" % (update.name)
                    for depend in depends_on:
                        email_body += u"* %s\n" % (depend.name)
                    email_body += u"\n"

        return email_body

    def _add_general_updates_as_json(self, general_updates):
        records = []
        for update in general_updates:
            general_update_object = {
                "name" : u"%s-%s-%s" % (update.name, update.version, update.release),
                "source": update.repository
            }
            self._add_dependents_as_json(general_update_object, update)
            self._add_changelogs_as_json(general_update_object, update)
            records.append(general_update_object)

        return records

    def _add_dependents_as_json(self, general_update_object, update):
        if self.include_depends_on and len(self.pkg_fetcher.get_what_depends_on(update.name)) > 0:
            general_update_object['required_by'] = [depend.name for depend in self.pkg_fetcher.get_what_depends_on(update.name)]

        return general_update_object

    def _add_changelogs_as_json(self, general_update_object, update):
        try:
            changelog = self.pkg_fetcher.get_package_changelog(update.name,update.version,update.release)
        except:
            changelog = None

        if changelog:
            log_text = changelog.decode('utf-8')
            try:
                general_update_object['changelog'] = u"%s" % log_text
            except:
                print "Problem dealing with changelog entry %s" % (changelog)
                raise

        return general_update_object

    def _add_changelogs_to_email(self, general_updates, email_body):
        if len(general_updates) > 0:
            changelogs = map(lambda pkg: { 'name': pkg.name, 'changelog': self.pkg_fetcher.get_package_changelog(pkg.name,pkg.version,pkg.release)},general_updates)
            email_body += u"\n\nChange logs for available package updates:\n\n"

            for update in general_updates:
                changelog_entry = next(cl for cl in changelogs if cl['name'] == update.name)
                # Some log text is in unicode
                log_text = changelog_entry['changelog'].decode('utf-8')
                try:
                    email_body += u"%s-%s-%s\n%s\n\n" % (update.name, update.version, update.release, log_text)
                except:
                    print "Problem dealing with changelog entry %s" % (changelog_entry)
                    raise

        return email_body

    def _handle_section_boundary(self, email_body):
        if email_body != u'':
            email_body += u"\n"
        return email_body

    def get_report_content(self):
        email_body = u''
        with self.db_session_fetch as session:
            self.annoyance_check = self.annoyance_fetcher.fetch(session)
            advisories = self._get_sorted_relevant_advisories()
            email_body = self._add_advisories_to_email(advisories, email_body)
            email_body = self._handle_section_boundary(email_body)
            general_updates = self._get_general_updates()
            email_body = self._add_general_updates_to_email(general_updates, email_body)
            email_body = self._handle_section_boundary(email_body)
            email_body = self._add_changelogs_to_email(general_updates, email_body)

        self.annoyance_check = None
        return email_body

    def get_report_content_as_json(self):
        output = {}
        with self.db_session_fetch as session:
            self.annoyance_check = self.annoyance_fetcher.fetch(session)
            output['advisories'] = self._get_advisories_as_json(self._get_sorted_relevant_advisories())
            output['updates'] = self._add_general_updates_as_json(self._get_general_updates())

        self.annoyance_check = None

        return json.dumps(output)


from package import Package
from errata_item import ErrataItem
from sqlalchemy import or_

class AnnoyanceCheck:
    def __init__(self, session):
        self.session = session
    
    def is_advisory_alert_necessary(self, advisory):
        existing = self.session.query(ErrataItem).filter(ErrataItem.advisory_id == advisory.advisory_id).all()          
        if len(existing) == 0:
            self.session.add(advisory)
            self.session.commit()
            return True
        else:
            return False
    
    def is_package_alert_necessary(self, package):        
        existing = self.session.query(Package).filter(Package.name == package.name).all()            
        not_yet_notified = any(existing_package_notification.compare_evr(package) < 0 for existing_package_notification in existing)
        if len(existing) == 0 or not_yet_notified == True:
            self.session.add(package)
            self.session.commit()
            return True
        else:
            return False
                
    def remove_old_alerts_for_package(self, package):
        old_versions = self.session.query(Package) \
        .filter(Package.name == package.name) \
        .filter(or_(Package.version != package.version, Package.release != package.release)) \
        .all()
        
        for old_ver in old_versions:
            self.session.delete(old_ver)
            
        self.session.commit()       

    def remove_old_advisories(self, active_advisories):
        notified_advisories = self.session.query(ErrataItem).all()
        active_advisory_ids = map(lambda advisory: advisory.advisory_id, active_advisories)
        old_advisories = filter(lambda notified_advisory: notified_advisory.advisory_id not in active_advisory_ids, notified_advisories)
        for old_advisory in old_advisories:
            self.session.delete(old_advisory)
            
        self.session.commit()


import re

class OsVersionFetcher:
    def get_complete_version(self):
        version_file = open('/etc/centos-release')
        try:
            version_string = version_file.read()
            return re.match(r'.*?(\d+\S+)\s.*',version_string).group(1)
        finally:
            version_file.close
        
    def get_top_level_version(self):
        complete = self.get_complete_version()
        return re.match(r'.*?(\d+)\..*',complete).group(1)
        
    def get_mid_level_version(self):
        complete = self.get_complete_version()
        return re.match(r'(\d+\.\d+).*', complete).group(1)



import subprocess

class MockableExecute:
    def run_command(self,command,command_input=None):
        proc = subprocess.Popen(command, stdout=subprocess.PIPE, stdin=subprocess.PIPE)
        out = proc.communicate(input=command_input)[0]
        return out


from rpmUtils.miscutils import splitFilename

class PackageParser:
    @staticmethod
    def parsePackage(filename):
        (n, v, r, e, a) = splitFilename(filename)
        return {
        'name':n,
        'version':v,
        'release':r,
        'arch':a
        }
    


import sqlalchemy
from sqlite3 import dbapi2 as sqlite
from sqlalchemy.orm import sessionmaker
from db_base import Base
import os

class db_session_fetcher:
    DEFAULT_DB_PATH = '/var/lib/centos-package-cron/already_annoyed.sqlite'
    
    def __init__(self, db_path=DEFAULT_DB_PATH):
        self.db_path = db_path
        self.session = None
        self.engine = None
        
    def __enter__(self):
        if self.engine == None:
            absolute_path = os.path.abspath(self.db_path)
            parent_dir = os.path.dirname(absolute_path)
            if not os.path.exists(parent_dir):
                raise Exception("Unable to find a parent directory for DB %s, did you install properly?" % (absolute_path))
                
            if not os.access(parent_dir, os.W_OK):
                raise Exception("Unable to write to directory for DB file %s, do you need to run as root?" % (absolute_path))
                
            if os.path.exists(absolute_path) and not os.access(absolute_path, os.W_OK):
                raise Exception("Unable to write to DB file %s, do you need to run as root?" % (absolute_path))
                
            self.engine = sqlalchemy.create_engine('sqlite:///%s' % (self.db_path),module=sqlite)
            
        Base.metadata.create_all(self.engine)
        Session = sessionmaker(bind=self.engine)
        self.session = Session()        
        return self.session
        
    def __exit__(self, type, value, traceback):
        self.session.close()
                


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import socket
import sys
reload(sys)
sys.setdefaultencoding('utf-8')
from report_producer import ReportProducer
from db_session_fetcher import db_session_fetcher
import smtplib
from email.mime.text import MIMEText
import pkg_resources

__VERSION__ = pkg_resources.require("centos_package_cron")[0].version

def main():
    args = parse_args()
    if args.output not in ['email', 'stdout']:
        print "%s is not valid for the -o/--output parameter. Must be email or stdout" % (args.output)
        raise

    repos_to_exclude_list = []
    if args.disablerepo is not None:
        repos_to_exclude_list = args.disablerepo.split(',')
    repos_to_include_list = []
    if args.enablerepo is not None:
        repos_to_include_list = args.enablerepo.split(',')

    skipold = not args.forceold
    producer = ReportProducer(repos_to_exclude_list, repos_to_include_list, skipold, args.skip_sqlite_file_path, include_depends_on=args.include_depends_on)

    report_content = producer.get_report_content_as_json() if args.json else producer.get_report_content()

    if report_content != '':
        if args.output == 'stdout':
            print report_content
        else:
            server = smtplib.SMTP('localhost')
            message = MIMEText(report_content.encode('utf-8'), 'plain', 'utf-8')
            message['Subject'] = args.email_subject
            message['From'] = args.email_from
            message['To'] = args.email_to
            server.sendmail(from_addr=args.email_from, to_addrs=[args.email_to], msg=message.as_string())
            server.quit

def parse_args():
    parser = argparse.ArgumentParser(description="Emails administrators with CentOS security updates and changelogs of non-security updates. Version %s" % __VERSION__, formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument('-e', '--email_to',
    type=str,
    default='root',
    help='Email following user with the output')

    parser.add_argument('-o', '--output',
    type=str,
    default='email',
    help='How should report be sent, email or stdout are valid values')

    parser.add_argument('-f', '--email_from',
    type=str,
    default="CentOS Update Check on %s <noreply@centos.org>" %(socket.gethostname()),
    help='Send the email from this user.')

    parser.add_argument('-s', '--email_subject',
    type=str,
    default="CentOS Update Check on %s" %(socket.gethostname()),
    help='Send the email using this subject')

    parser.add_argument('-dr','--disablerepo',
    type=str,
    help='List of comma separated repos to exclude when dealing with Yum')

    parser.add_argument('-er','--enablerepo',
    type=str,
    help='List of comma separated repos to include when dealing with Yum')

    parser.add_argument('-fo','--forceold',
    help='Instead of the default behavior to only complain once for a given advisory/package update notice, repeats them with each run.',
    action="store_true")

    parser.add_argument('-db','--skip-sqlite-file-path',
    type=str,
    default=db_session_fetcher.DEFAULT_DB_PATH,
    help='The location of the Sqlite DB used to track which notifications you have already received.')

    parser.add_argument('-do','--include-depends-on',
    help='When a package update is listed, show what packages on your system depend on that package',
    action="store_true")

    parser.add_argument('-j','--json',
    help='Output the results in JSON format',
    action="store_true")

    return parser.parse_args()

if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python

import sys
import setuptools

from setuptools.command.test import test as TestCommand

class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)

version = '1.0.10'
setuptools.setup(name='centos_package_cron',
      version=version,
      description='CentOS Package Update Utilities',
      author='Brady Wied',
      author_email='support@bswtechconsulting.com',
      url='https://github.com/wied03/centos-package-cron',
      download_url='https://github.com/wied03/centos-package-cron/archive/releases/%s.tar.gz' % (version),
      license='BSD 2-Clause',
      packages=setuptools.find_packages(),
      setup_requires="pytest",
      install_requires=['sqlalchemy'],
      tests_require="mock",
      cmdclass = {'test': PyTest},
	  entry_points = {
	        'console_scripts': ['centos-package-cron=centos_package_cron.command_line:main'],
	  }
     )



import os
import sys
import yum
import re
import mockable_execute
from package import Package
from package_parser import PackageParser
        
class ChangeLogParser:
    def get_log_version_num_with_release_suffix(self,version,release):
        match = re.match(r'^.*?(\S+)\.el\S+\.(\d+)$', release)
        return version + '-' + match.group(1) + '.' + match.group(2) if match else None
    
    def get_log_version_num_without_release_suffix(self,version,release):
        match = re.match(r'^.*?(\S+)\.el\S+$', release)
        return version + '-' + match.group(1)  if match else None
    
    def get_log_version_nums(self,version,release): 
        versions = [self.get_log_version_num_with_release_suffix(version,release),
        self.get_log_version_num_without_release_suffix(version,release),
        version + '-' + release]
        return list(filter(lambda v: v != None,versions))
                    
    def get_regex_patterns(self,name,version,release):
        versions = self.get_log_version_nums(version,release)
        escaped_package_name = re.escape(name)
        pattern_former = lambda v: r'.*^(?:\d+\:){0,1}' + escaped_package_name +r'.*?(^\*.*? ' + re.escape(v) + r'.*?)^\*.*'        
        patterns = map(pattern_former,versions)
        return patterns
    
    def get_regexes(self,name,version,release):
        return map(lambda pat: re.compile(pat,re.MULTILINE | re.DOTALL), self.get_regex_patterns(name,version,release))     
    
    def parse(self,output,name,version,release):
        regexes = self.get_regexes(name,version,release)
        matches = map(lambda r: r.match(output), regexes)
        match = next((match for match in matches if match != None), None)   
        if match == None:
            return "Unable to parse changelog for package %s version %s release %s" % (name,version,release)
        return match.group(1)

class PackageFetcher:
    def __init__(self,changelog_parser,executor,repos_to_exclude=[],repos_to_include=[]):
        self.changelog_parser = changelog_parser
        self.yb = yum.YumBase()
        self.yb.setCacheDir()
        self.executor = executor
        self.repos_to_exclude = repos_to_exclude
        self.repos_to_include = repos_to_include
        for repo in repos_to_exclude:
            self.yb.repos.disableRepo(repo)
        for repo in repos_to_include:
            self.yb.repos.enableRepo(repo)
    
    def fetch_installed_packages(self):     
        packages = self.yb.rpmdb.returnPackages()
        result = map(lambda x: Package(x.name,x.version,x.release, x.arch, x.repoid), packages)
        return result

    def get_package_updates(self):
        raw_updates = self.yb.update()
        result = map(lambda x: Package(x.name,x.version,x.release, x.arch, x.repoid), raw_updates)
        return result
        
    def get_what_depends_on(self,name):
        command = ['rpm', '-q', '--provides', name]
        output = self.executor.run_command(command)        
        command = ['sed', r'''s/^/"/;s/\([^[:space:]]\) *$/\1"/;/=/{h;s/ =.*$/"/;G}''']
        output = self.executor.run_command(command, command_input=output)
        command = ['xargs', 'rpm', '-q', '--whatrequires']
        output = self.executor.run_command(command, command_input=output)
        command = ['grep', '-v', '-E', r'''^no package''']
        packages = self.executor.run_command(command, command_input=output)
        packages = packages.split("\n")
        packages = sorted(packages)
        packages = list(set(packages))
        packages = list(filter(lambda v: v != "",packages))
        packages = map(lambda pkgStr: PackageParser.parsePackage(pkgStr), packages)
        return map(lambda pkgHsh: Package(pkgHsh['name'], pkgHsh['version'], pkgHsh['release'], pkgHsh['arch'], repository=""), packages)
        
    def get_package_changelog(self,name,version,release):
        args = ['/usr/bin/yum']
        if len(self.repos_to_exclude) > 0:
            repos_flat = "--disablerepo=%s" % (','.join(self.repos_to_exclude))
            args.append(repos_flat)
        if len(self.repos_to_include) > 0:
            repos_flat = "--enablerepo=%s" % (','.join(self.repos_to_include))
            args.append(repos_flat)
        args += ['changelog', 'updates', name]                  
        output = self.executor.run_command(args)
        return self.changelog_parser.parse(output,name,version,release)
        